@@ -20,27 +20,10 @@ import mlflow
 import pandas as pd
 import prophet
 
-OFFSET_ALIAS_MAP = {
-    "W": "W",
-    "d": "D",
-    "D": "D",
-    "days": "D",
-    "day": "D",
-    "hours": "H",
-    "hour": "H",
-    "hr": "H",
-    "h": "H",
-    "H": "H",
-    "m": "min",
-    "minute": "min",
-    "min": "min",
-    "minutes": "min",
-    "T": "T",
-    "S": "S",
-    "seconds": "S",
-    "sec": "S",
-    "second": "S"
-}
+from databricks.automl_runtime.forecast import OFFSET_ALIAS_MAP
+from mlflow.exceptions import MlflowException
+from mlflow.models.signature import infer_signature, ModelSignature
+from mlflow.protos.databricks_pb2 import INVALID_PARAMETER_VALUE
 
 PROPHET_CONDA_ENV = {
     "channels": ["conda-forge"],
@@ -49,7 +32,7 @@ PROPHET_CONDA_ENV = {
             "pip": [
                 f"prophet=={prophet.__version__}",
                 f"cloudpickle=={cloudpickle.__version__}",
-                f"databricks-automl-runtime==0.2.0"
+                f"databricks-automl-runtime==0.2.5",
             ]
         }
     ],
@@ -61,19 +44,34 @@ class ProphetModel(mlflow.pyfunc.PythonModel):
     """
     Prophet mlflow model wrapper for univariate forecasting.
     """
-    def __init__(self, model_json: Union[Dict[str, str], str], horizon: int, frequency: str) -> None:
+
+    def __init__(self, model_json: Union[Dict[str, str], str], horizon: int, frequency: str,
+                 time_col: str) -> None:
         """
         Initialize the mlflow Python model wrapper for mlflow
         :param model_json: json string of the Prophet model or
         the dictionary of json strings of Prophet model for multi-series forecasting
         :param horizon: Int number of periods to forecast forward.
         :param frequency: the frequency of the time series
+        :param time_col: the column name of the time column
         """
         self._model_json = model_json
         self._horizon = horizon
         self._frequency = frequency
+        self._time_col = time_col
         super().__init__()
 
+    def _validate_cols(self, df: pd.DataFrame, required_cols: List[str]):
+        df_cols = set(df.columns)
+        required_cols_set = set(required_cols)
+        if not required_cols_set.issubset(df_cols):
+            raise MlflowException(
+                message=(
+                    f"Input data columns '{list(df_cols)}' do not contain the required columns '{required_cols}'"
+                ),
+                error_code=INVALID_PARAMETER_VALUE,
+            )
+
     def load_context(self, context: mlflow.pyfunc.model.PythonModelContext) -> None:
         """
         Loads artifacts from the specified :class:`~PythonModelContext` that can be used
@@ -117,23 +115,35 @@ class ProphetModel(mlflow.pyfunc.PythonModel):
         """
         return self._predict_impl(horizon)
 
-    def predict(self, context: mlflow.pyfunc.model.PythonModelContext, input: pd.DataFrame) -> pd.DataFrame:
+    def predict(self, context: mlflow.pyfunc.model.PythonModelContext, model_input: pd.DataFrame) -> pd.Series:
         """
         Predict API from mlflow.pyfunc.PythonModel
         :param context: A :class:`~PythonModelContext` instance containing artifacts that the model
                         can use to perform inference.
-        :param input: Input dataframe
+        :param model_input: Input dataframe
         :return: A pd.DataFrame with the forecast components.
         """
-        return self._predict_impl()
+        self._validate_cols(model_input, [self._time_col])
+        test_df = pd.DataFrame({"ds": model_input[self._time_col]})
+        predict_df = self.model().predict(test_df)
+        return predict_df["yhat"]
+
+    def infer_signature(self, sample_input: pd.DataFrame = None) -> ModelSignature:
+        if sample_input is None:
+            sample_input = self._make_future_dataframe(horizon=1)
+            sample_input.rename(columns={"ds": self._time_col}, inplace=True)
+        signature = infer_signature(sample_input, self.predict(context=None, model_input=sample_input))
+        return signature
 
 
 class MultiSeriesProphetModel(ProphetModel):
     """
     Prophet mlflow model wrapper for multi-series forecasting.
     """
+
     def __init__(self, model_json: Dict[str, str], timeseries_starts: Dict[str, pd.Timestamp],
-                 timeseries_end: str, horizon: int, frequency: str) -> None:
+                 timeseries_end: str, horizon: int, frequency: str, time_col: str, id_cols: List[str],
+                 ) -> None:
         """
         Initialize the mlflow Python model wrapper for mlflow
         :param model_json: the dictionary of json strings of Prophet model for multi-series forecasting
@@ -141,11 +151,14 @@ class MultiSeriesProphetModel(ProphetModel):
         :param timeseries_end: the end time of the time series
         :param horizon: Int number of periods to forecast forward
         :param frequency: the frequency of the time series
+        :param time_col: the column name of the time column
+        :param id_cols: the column names of the identity columns for multi-series time series
         """
-        super().__init__(model_json, horizon, frequency)
+        super().__init__(model_json, horizon, frequency, time_col)
         self._frequency = frequency
         self._timeseries_end = timeseries_end
         self._timeseries_starts = timeseries_starts
+        self._id_cols = id_cols
 
     def model(self, id: str) -> prophet.forecaster.Prophet:
         """
@@ -218,27 +231,40 @@ class MultiSeriesProphetModel(ProphetModel):
         :param horizon: Int number of periods to forecast forward.
         :return: A pd.DataFrame with the forecast components.
         """
-        forecast_df = self._predict_impl(df,  horizon)
+        forecast_df = self._predict_impl(df, horizon)
         return_cols = self.get_reserved_cols() + ["ds", "ts_id"]
         result_df = pd.DataFrame(columns=return_cols)
         result_df = pd.concat([result_df, forecast_df])
         result_df["ts_id"] = str(df["ts_id"].iloc[0])
         return result_df[return_cols]
 
-    def predict(self, context: mlflow.pyfunc.model.PythonModelContext, input_df: pd.DataFrame) -> pd.DataFrame:
+    def predict(self, context: mlflow.pyfunc.model.PythonModelContext, model_input: pd.DataFrame) -> pd.Series:
         """
         Predict API from mlflow.pyfunc.PythonModel
         :param context: A :class:`~PythonModelContext` instance containing artifacts that the model
                         can use to perform inference.
-        :param input_df: Input dataframe
+        :param model_input: Input dataframe
         :return: A pd.DataFrame with the forecast components.
         """
-        return self._predict_impl(input_df)
+        self._validate_cols(model_input, self._id_cols + [self._time_col])
+        test_df = model_input.copy()
+        test_df["ts_id"] = test_df[self._id_cols].astype(str).agg('-'.join, axis=1)
+        test_df.rename(columns={self._time_col: "ds"}, inplace=True)
+        predict_df = test_df.groupby("ts_id").apply(lambda df: self.model(df.name[0]).predict(df)).reset_index()
+        return_df = test_df.merge(predict_df, how="left", on=["ts_id", "ds"])
+        return return_df["yhat"]
 
 
-def mlflow_prophet_log_model(prophet_model: Union[ProphetModel, MultiSeriesProphetModel]) -> None:
+def mlflow_prophet_log_model(prophet_model: Union[ProphetModel, MultiSeriesProphetModel],
+                             sample_input: pd.DataFrame = None) -> None:
     """
     Log the model to mlflow
     :param prophet_model: Prophet model wrapper
+    :param sample_input: sample input Dataframes for model inference
     """
-    mlflow.pyfunc.log_model("model", conda_env=PROPHET_CONDA_ENV, python_model=prophet_model)
+    # log the model without signature if infer_signature is failed.
+    try:
+        signature = prophet_model.infer_signature(sample_input)
+    except Exception: # noqa
+        signature = None
+    mlflow.pyfunc.log_model("model", conda_env=PROPHET_CONDA_ENV, python_model=prophet_model, signature=signature)
